﻿using System.Collections;
using System.Collections.Specialized;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using Infragistics.Controls.Charts;

namespace WpfApplication1.Views.Gauges
{
    public class ScaleBindingInfo : FrameworkElement
    {
        public static DependencyProperty OwnerProperty =
            DependencyProperty.Register("Owner",
                                        typeof(Control), typeof(ScaleBindingInfo),
                                        new PropertyMetadata(null,
                                                             (o, e) =>
                                                             {
                                                                 (o as ScaleBindingInfo).OnOwnerChanged(e);
                                                             }));

        public Control Owner
        {
            get { return (Control)GetValue(OwnerProperty); }
            set { SetValue(OwnerProperty, value); }
        }

        public static DependencyProperty ItemsSourceProperty =
            DependencyProperty.Register("ItemsSource",
                                        typeof(IEnumerable), typeof(ScaleBindingInfo),
                                        new PropertyMetadata(null,
                                                             (o, e) =>
                                                             {
                                                                 (o as ScaleBindingInfo).OnItemsSourceChanged(e);
                                                             }));

        public IEnumerable ItemsSource
        {
            get { return (IEnumerable)GetValue(ItemsSourceProperty); }
            set { SetValue(ItemsSourceProperty, value); }
        }

        protected void OnItemsSourceChanged(
            DependencyPropertyChangedEventArgs e)
        {
            Clear();

            if (e.OldValue != null)
            {
                UnRegCollection(e.OldValue as IEnumerable);
            }
            if (e.NewValue != null)
            {
                RegCollection(e.NewValue as IEnumerable);
                UpdateFromCollection(e.NewValue as IEnumerable);
            }
        }

        public static DependencyProperty OuterContextProperty =
            DependencyProperty.Register("OuterContext",
                                        typeof(object), typeof(ScaleBindingInfo),
                                        new PropertyMetadata(null,
                                                             (o, e) =>
                                                             {
                                                             }));

        public object OuterContext
        {
            get { return GetValue(OuterContextProperty); }
            set { SetValue(OuterContextProperty, value); }
        }

        private void OnOwnerChanged(
            DependencyPropertyChangedEventArgs e)
        {
            SetBinding(DataContextProperty,
                       new Binding("OuterContext")
                       {
                           Source = this
                       });
            Clear();
            UpdateFromCollection(ItemsSource);
        }

        private void Clear()
        {
            if (Owner == null)
            {
                return;
            }
        }

        private void UpdateFromCollection(
            IEnumerable collection)
        {
            if (collection == null)
            {
                return;
            }
            foreach (object item in collection)
            {
                AddItem(item);
            }
        }


        public static void AddGaugeElements( XamRadialGauge gauge )
        {
            var s = new RadialGaugeScale
            {
                StartValue = 0,
                EndValue = 140,
            };
            var l = new RadialGaugeLabelGroup
            {
                Interval = 20
            };
            var n = new RadialGaugeNeedle { Value = 33 };

            gauge.Scales.Add(s);
            s.Needles.Add(n);
            s.LabelGroups.Add(l);            
        }

        private void AddItem(
            object item)
        {
            AddGaugeElements((XamRadialGauge)Owner);
        }

        private void RegCollection(
            IEnumerable collection)
        {
            if (collection is INotifyCollectionChanged)
            {
                (collection as INotifyCollectionChanged).CollectionChanged
                    += ScaleBindingInfo_CollectionChanged;
            }
        }

        private void UnRegCollection(
            IEnumerable collection)
        {
            if (collection is INotifyCollectionChanged)
            {
                (collection as INotifyCollectionChanged).CollectionChanged
                    -= ScaleBindingInfo_CollectionChanged;
            }
        }

        private void ScaleBindingInfo_CollectionChanged(
            object sender,
            NotifyCollectionChangedEventArgs e)
        {
            if (Owner == null)
            {
                return;
            }
            //this could be made more efficient of course.     
            Clear();
            UpdateFromCollection(ItemsSource);
        }
    }
}
